package com.jpattern.gwt.client.view;

import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HasWidgets;
import com.jpattern.gwt.client.presenter.IPresenter;

/**
 * 
 * A View based on the gwt Composite component
 * 
 * @author Francesco Cina'
 *
 * 20 Apr 2011
 */
public abstract class AGwtCompositeView<T extends IPresenter> extends Composite implements IView {

	private T presenter;
	private IErrorArea errorArea;
	
	@Override
	public final void render(HasWidgets container) {
		if (container!=null) {
		    container.clear();
		    container.add(this);
		}
	}
	

	@SuppressWarnings("unchecked")
	@Override
	public final void visit(IPresenter presenter) {
		this.presenter = (T) presenter;
	}

	@Override
	public final T getPresenter() {
		return presenter;
	}

	@Override
	public final IErrorArea getErrorArea() {
		if (errorArea == null) {
			errorArea = buildErrorArea();
		}
		return errorArea;
	}

	@Override
	public final void onUnload() { 
		onUnloadView();
		getPresenter().onOnloadPresenter();
	}
	
	
	
	protected abstract void onUnloadView();
	
	protected abstract IErrorArea buildErrorArea();
	
}
